@@ -19,8 +19,8 @@
 ;; URL:      https://github.com/meedstrom/org-mem
 ;; Created:  2025-03-15
 ;; Keywords: text
-;; Package-Version: 20260211.117
-;; Package-Revision: 3f9c03039684
+;; Package-Version: 20260217.2038
+;; Package-Revision: 9c7f3e1a6241
 ;; Package-Requires: ((emacs "29.1") (el-job "2.7.1") (llama "0.5.0"))
 
 ;;; Commentary:
@@ -42,7 +42,7 @@
 ;;; Code:
 
 (define-obsolete-variable-alias 'org-mem--bump-int 'org-mem-internal-version "2026-01-27 (after 0.26.4)")
-(defconst org-mem-internal-version 30 "Not a version number, but bumped sometimes.")
+(defconst org-mem-internal-version 32 "Not a version number, but bumped sometimes.")
 
 (require 'cl-lib)
 (require 'subr-x)
@@ -200,9 +200,6 @@ These regions will not be scanned for links nor active timestamps."
   :type '(alist :key-type regexp :value-type regexp)
   :package-version '(org-mem . "0.23.0"))
 
-(defvar org-mem-scratch nil
-  "Work buffer held current while executing some hooks.")
-
 
 ;;; Basics
 
@@ -313,7 +310,7 @@ Note: All tables cleared often, meant for memoizations."
   (tags-inherited        nil :read-only t :type list)
   (tags-local            nil :read-only t :type list)
   (todo-state            nil :read-only t :type string-or-nil)
-  (pseudo-id             -1  :read-only t :type integer        :documentation "Generated by `org-mem-parser--mk-id'.  Not `eq'-safe, so use `=', `eql' or `equal'."))
+  (pseudo-id             -1  :read-only t :type integer        :documentation "Generated by `org-mem-parser--mk-id'.  Not `eq'-safe, use `=', `eql' or `equal'."))
 
 
 ;;; To find objects to operate on
@@ -822,7 +819,7 @@ See more info at `org-mem-entry-file'."
 
 (defun org-mem-file-attributes (file/entry/link)
   "The cached `file-attributes' list for file at FILE/ENTRY/LINK.
-The uid and gid are strings, see ID-FORMAT in `file-attributes'."
+The uid and gid are integers, see ID-FORMAT in `file-attributes'."
   (nth 1 (org-mem--get-file-metadata file/entry/link)))
 
 (defun org-mem-file-line-count (file/entry/link)
@@ -1058,10 +1055,7 @@ These link-targets are determined by `org-mem--split-roam-refs-field'."
   "Extract valid components of a ROAM-REFS field.
 What is valid?  See \"org-mem-test.el\"."
   (when roam-refs
-    (with-current-buffer (if (eq (current-buffer) org-mem-scratch)
-                             org-mem-scratch
-                           (setq org-mem-scratch
-                                 (get-buffer-create " *org-mem-scratch*" t)))
+    (with-current-buffer (get-buffer-create " *org-mem-scratch*" t)
       (erase-buffer)
       (insert roam-refs)
       (goto-char 1)
@@ -1278,7 +1272,7 @@ overrides a default message printed when `org-mem-do-cache-text' is t."
         (el-job-ng-run :id 'org-mem
                        :inject-vars (append (org-mem--mk-work-vars)
                                             (el-job-ng-vars org-mem-inject-vars))
-                       :require (append '(org-mem-parser) org-mem-load-features)
+                       :require (cons 'org-mem-parser org-mem-load-features)
                        :eval org-mem-eval-forms
                        :inputs files
                        :funcall-per-input #'org-mem-parser--parse-file
@@ -1300,7 +1294,8 @@ overrides a default message printed when `org-mem-do-cache-text' is t."
                        (erase-buffer)
                        (insert-file-contents file)
                        (puthash file (buffer-string) org-mem--truename<>content))))))
-            (eval `(let ,(cl-loop for (var . val) in org-mem-inject-vars
+            (eval `(let ,(cl-loop for (var . val)
+                                  in (el-job-ng-vars org-mem-inject-vars)
                                   if (listp val)
                                   collect `(,var ',val)
                                   else collect `(,var ,val))
@@ -1320,12 +1315,12 @@ overrides a default message printed when `org-mem-do-cache-text' is t."
   (clrhash org-mem--truename<>entries)
   (clrhash org-mem--pseudo-id<>links)
   (setq org-mem--title-collisions nil)
-  (let (bad-paths problems)
-
+  (let ((bad-paths (cl-loop for (bad) in parse-results when bad collect bad))
+        problems)
+    (org-mem--invalidate-file-names bad-paths)
     ;; Build tables.
-    (with-current-buffer (setq org-mem-scratch (get-buffer-create " *org-mem-scratch*" t))
-      (cl-loop for (bad-path problem file-data entries links) in parse-results do
-               (when bad-path (push bad-path bad-paths))
+    (with-current-buffer (get-buffer-create " *org-mem-scratch*" t)
+      (cl-loop for (_ problem file-data entries links) in parse-results do
                (when problem (push problem problems))
                (puthash (car file-data) file-data org-mem--truename<>metadata)
                (run-hook-with-args 'org-mem--record-file-functions file-data)
@@ -1336,7 +1331,6 @@ overrides a default message printed when `org-mem-do-cache-text' is t."
                  (push link (gethash (org-mem-link-entry-pseudo-id link)
                                      org-mem--pseudo-id<>links))
                  (run-hook-with-args 'org-mem--record-link-functions link))))
-    (org-mem--invalidate-file-names bad-paths)
     (org-mem--rebuild-specially-indexed-tables)
 
     ;; Message on interactive reset.
@@ -1782,11 +1776,13 @@ This is a transition helper.  Org-mem 0.27.0 changed the style of
 PARSE-RESULTS passed to `org-mem-pre-full-scan-functions' & co.
 
 For reference, the new style is a list as long as the number of files,
-each element being a list of 5 lists:
+each element being a list of 5 elements:
 
    \((BAD-PATH PROBLEM FILE-DATUM ENTRIES LINKS) ...)
 
-Old style was single list of 5 lists:
+This data represents one file.
+
+Old style was single list of 5 lists, representing all files combined:
 
    \(BAD-PATHS FILE-DATA ENTRIES LINKS PROBLEMS)"
   (cl-loop
